import logging
import os
import subprocess

# We must install mkvtools and add it to Path in order for it to work.

VIDEO_EXTENSIONS = {'.mkv', '.mp4', '.avi', '.flv'}

def getSubtitlePath(input_path: str) -> (str,str):
    """This function returns a tuple of output path name and subtitle path name
    
    Arguments:
        input_path {str} -- The path name of original video file
    
    Returns:
        (str,str) -- A tuple (subtitle path name, output file path name)
    """
    logging.debug(f"Removing the extension to extract the path name.")
    episode_name = input_path[:input_path.rfind('.')]
    subtitle_path = f"{episode_name}.srt"
    output_path = f"{episode_name}embedded.mkv"
    if os.path.isfile(subtitle_path):
        logging.info(f"Subtitle file found for {episode_name}.")
    else:
        logging.info(f"No subtitle file found for {episode_name}.")
        subtitle_path = ''
    return (subtitle_path,output_path)

def clean(input_path:str,subtitle_path:str, rm_orig=True):
    """This function removes the 
    
    Arguments:
        input_path {str} -- The path name of the original video file
        subtitle_path {str} -- The path name of the subtitle file
    
    Keyword Arguments:
        rm_orig {bool} -- argument to check if function needs to be executed. (default: {True})
    """
    if rm_orig:
        logging.info(f"Removing the video file {input_path}")
        os.remove(input_path)
        logging.info(f"Removing the subtitle file {subtitle_path}")
        os.remove(subtitle_path)
        if os.path.isfile(subtitle_path) or os.path.isfile(input_path):
            logging.error(f"Cleanup Failed. Files still exists")
        else:
            logging.info(f"Cleanup Successfull")

def executeMKVMerge(input_path:str,subtitle_path:str,output_path:str,rm_orig=False):
    """This function executes the mkvmerge script using a subprocess. It may also clean up the original files if rm_orig is True
    
    Arguments:
        input_path {str} -- The input path of the episode
        subtitle_path {str} -- The path name of the subtitle file
        output_path {str} -- The path name of the output file
    
    Keyword Arguments:
        rm_orig {bool} -- argument to check if original need to be removed. (default: {False})
    """
    logging.debug(f"Embedding Subtitles from {subtitle_path} into {input_path}")
    mkvmerge_command = ['mkvmerge','-o', output_path, input_path, subtitle_path]
    if subprocess.run(mkvmerge_command).returncode==0:
        logging.info(f"Embedding Succesfull")
        clean(input_path,subtitle_path,rm_orig)
        logging.info(f"Removing any mention of embedding.")
        os.rename(output_path,f"{output_path[-12:]}.mkv")
    else:
        logging.error(f"Embedding Failed")

def embedSubtitles(episode_path:str, rm_orig=False):
    """This function takes the path of episode and sees if there is a matching subtitle file.
    If there is subtitle file near it with same name, it will merge it and output a single mkvfile.
    
    Arguments:
        episode_path {str} -- Path name of the Episode.
    
    Keyword Arguments:
        rm_orig {bool} -- argument to check if original need to be removed. (default: {False})
    """
    if episode_path[-4:] not in VIDEO_EXTENSIONS:
        logging.warning(f"{episode_path} not a video and therefore skipped.")
    else:
        logging.info(f"Selected file {episode_path}")                
        (subtitle_path,output_path) = getSubtitlePath(episode_path)
        if subtitle_path!='':
            executeMKVMerge(episode_path,subtitle_path,output_path,rm_orig)


import setuptools
with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
     name='EpisodeFixer',  
     version='1.0.4',
     packages=['EpisodeFixer'] ,
     entry_points={'console_scripts': ['EpisodeFixer = EpisodeFixer.EpisodeFixer:main']},
     author="Vasu Jain",
     author_email="jain.vasu.631@gmail.com",
     description="An Episode Fixer which can embed subtitles and rename episodes.",
     long_description=long_description,
     long_description_content_type="text/markdown",
     url="https://github.com/jainvasu631/EpisodeFixer",
     download_url="https://github.com/jainvasu631/EpisodeFixer/archive/v1.0.4.tar.gz",
     install_requires=['bs4'],
     classifiers=[
         "Programming Language :: Python :: 3",
         "Programming Language :: Python :: 3.6",
         "Programming Language :: Python :: 3.7",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
         "Natural Language :: English",
     ],
 )


# Contains all functions requiring User Input

import logging
from .Parsers import parseSelection
from typing import Set

def getLogLevel() -> str:
    """This functions get the log level from the user and configures the logging level
    
    Returns:
        str -- Logging Level Selected by User. It can be WARNING, DEBUG or INFO(default)
    """
    logLevel = input("Please enter the Logging Level. Options are DEBUG(For Debugging), INFO(To see what's going on)[Default] or WARNING(Only see errors or problems): ")
    if (logLevel == "WARNING"):
        logging.basicConfig(level=logging.WARNING)
    elif (logLevel == "DEBUG"):
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)
    
    logging.info(f"User entered {logLevel} .")

    return logLevel

#get Output format from the user.
def getOutputFormat()-> str:
    """This functions takes in the output format from the user
    
    Returns:
        str -- The output format enterred by the user.
    """
    
    print("""The output format options are :
    SE -> S01E01. This is default.
    se -> s01e01
    X -> 01X01
    x -> 01x01
    0 -> 0101
    v -> season 01 episode 01
    V -> Season 01 - Episode 01""")
    
    outputFormat = input("Please enter the output format for Episode Filenames. Press [ENTER] to Go with default:")
    logging.info(f"User entered {outputFormat} .")
    return outputFormat

def getShowTitle() -> str:
    """This functions takes in show title from user
    
    Returns:
        str -- The show titles as input by the User
    """
    showTitle = input("Please enter the name of the TV Show : ")
    logging.info(f"User entered {showTitle} .")
    return showTitle

def getShowDirectory() -> str:
    """This function takes in show directory from user
    
    Returns:
        str -- The show directory as input by the User
    """
    showDirectory = input("Please enter the parent directory containing the TV Show Seasons : ")
    logging.info(f"User entered {showDirectory} .")
    return showDirectory

def userInput()-> dict:
    """The purpose of this function is to take User Input for Parameters. 
    It stores them in a dict which is passed on to function main.
    
    Returns:
        dict -- The dict contains the UserInput like log level, format, show name and show directory.
    """
    print("Welcome to Episode Fixer.")
    return {'Logging Level':getLogLevel(),'Format':getOutputFormat(),'Show Name':getShowTitle(),'Directory':getShowDirectory()}

def getParams()-> Set[str]:
    """
    This function asks the rename Parameters. 
    The user specifies the episodes which are to be renamed and which are to be skipped.
    It also confirms before asking for a rename. This ensures that we can skip a rename if we don't want it.  
    This is currently done by string formatting. In future implementations parsers may be used instead.
    
    Returns:
        Set[str] -- The set of episodes which needs to be processed.
    """

    print("""Specify the episodes.
    The format is : S01, S02E01, S01E02-05, S01-12, S01-12E11-24 / S05, S03E01, S03E02-05, S07-12, S06-12E17-24
    The episodes after '/' specify the episodes to be skipped.
    Press [ENTER] to include all episodes""")

    user_input = input("Please Input the Selection Range:")
    logging.info(f"User Enterted : {user_input}")
    logging.debug(f"Parsing the Selection Range")
    selected_episodes = parseSelection(user_input)
    logging.debug(f"Episodes in Selection Range are : {selected_episodes}")
    return selected_episodes

def getConfirmation(process:str='Process') -> bool:
    """This function gets the confirmation from the user for process.
    
    Keyword Arguments:
        process {str} -- The process which needs to be confirmed. e.g. Embedding/Renaming etc (default: {'Process'})
    
    Returns:
        bool -- If User Confirms we return True else False
    """
    confirmation = input(f"FINAL CONFIRMATION : {process} The Episodes? Press [Enter] to proceed. Any other Key to Skip : ")
    if confirmation:
        logging.info(f"{process} Aborted. User Typed {confirmation}")
        return False
    else:
        return True